#!/usr/bin/perl -w

#
# Program Summary:
#
# Name:             dailystrips.pl
# Description:      creates an HTML page containing a number of online comics, with an easily exensible framework
# Author:           Andrew Medico <amedico@calug.net>
# Created:          23 Nov 2000, 23:33
# Last Modified:    24 Jan 2001, 18:39
# Current Revision: 1.0.2
#

# Set up
use strict;

use HTTP::Request;
use LWP::UserAgent;
use POSIX qw(strftime);

my (%options, $version, @localtime, $long_date, $short_date, @get, @strips);

$version = "1.0.2";

@localtime = localtime;
$long_date = strftime("\%A \%B %-e, \%Y", @localtime);
$short_date = strftime("\%Y.\%m.\%d", @localtime);


# Parse options
for (@ARGV)	{
	if ($_ eq "" or $_ =~ m/^(--help|-h)$/o) {
		print "Usage: $0 [OPTION] STRIPS\n";
		print "'all' may be used to retrieve all known strips,\n";
		print "or use option --list to list available strips\n";
		print "\nOptions:\n";
		print "      --output=FILE     outputs HTML to FILE instead of STDOUT\n";
		print "  -l  --local           outputs HTML to file and saves strips locally\n";
		print "      --noindex         disables symlinking current page to index.html\n";
		print "  -a  --archive         generates archive.html as a list of all days,\n";
		print "                        when in local mode\n";
		print "  -q  --quiet           turns off progress messages\n";
		print "  -s  --save            if it appears that a particular strip has been\n";
		print "                        downloaded, does not attempt to re-download it";
		print "\nBugs and comments to amedico\@calug.net\n";
		exit;
	} elsif ($_ =~ m/^--list$/o) {
		print "Available strips:\n";
		print "adamathome            Adam\@Home\n";
		print "badtech               Badtech\n";		
		print "baldo                 Baldo\n";
		print "bizarro               Bizarro\n";
		print "thebigpicture         The Big Picture\n";
		print "boondocks             The Boondocks\n";
		print "calvinandhobbes       Calvin and Hobbes\n";
		print "cathy                 Cathy\n";
		print "citizendog            Citizen Dog\n";
		print "closetohome           Close to Home\n";
		print "cornered              Cornered\n";
		print "crabbyroad            Crabby Road\n";
		print "crankshaft            Crankshaft\n";
		print "dilbert               Dilbert\n";		
		print "doonesbury            Doonesbury\n";
		print "duplex                Duplex\n";
		print "thefifthwave          The 5th Wave\n";
		print "foxtrot               FoxTrot\n";
		print "thefuscobrothers      The Fusco Brothers\n";
		print "garfield              Garfield\n";
		print "heartofthecity        Heart of the City\n";
		print "inthebleachers        In the Bleachers\n";
		print "james                 James\n";
		print "joyoftech             The Joy of Tech\n";		
		print "misterboffo           Mister Boffo\n";
		print "nonsequitur           Non Sequitur\n";
		print "oddlyenough           Oddly Enough\n";
		print "overboard             Overboard\n";
		print "pennyarcade           Penny Arcade\n";		
		print "reallifeadventures    Real Life Adventures\n";
		print "redmeat               Redmeat\n";		
		print "reynoldsunwrapped     Reynolds Unwrapped\n";
		print "sinfest               Sinfest\n";		
		print "stonesoup             Stone Soup\n";
		print "tankmcnamara          Tank McNamara\n";
		print "tomthedancingbug      Tom the Dancing Bug\n";
		print "userfriendly          User Friendly\n";		
		print "ziggy                 Ziggy\n";
		print "all                   All strips\n";
		exit;
	} elsif ($_ =~ m/^(--archive|-a)$/o) {
		$options{'make_archive'} = 1;
	} elsif ($_ =~ m/^(--quiet|-q)$/o) {
		$options{'quiet'} = 1;
	} elsif ($_ =~ m/^(--save|-s)$/o) {
		$options{'save_existing'} = 1;
	} elsif ($_ =~ m/^--output=(.*)$/o) {
		$options{'output_file'} = "$1";
	} elsif ($_ =~ m/^(adamathome|badtech|baldo|bizarro|thebigpicture|boondocks|calvinandhobbes|cathy|citizendog|closetohome|cornered|crabbyroad|crankshaft|dilbert|doonesbury|duplex|thefifthwave|foxtrot|thefuscobrothers|garfield|heartofthecity|inthebleachers|james|joyoftech|misterboffo|nonsequitur|oddlyenough|overboard|pa|reallifeadventures|redmeat|reynoldsunwrapped|sinfest|stonesoup|tankmcnamara|tomthedancingbug|uf|ziggy|all)$/io) {
		push(@get, $_);
	} elsif ($_ =~ m/^(--local|-l)$/o) {
		$options{'local_mode'} = 1;
	} elsif ($_ =~ m/^--noindex$/o) {
		$options{'no_index'} = 1;
	} else {
		die "Unknown option: $_\n";
	}
}

unless (@get) {
	die "Error: no strip specified (--list to list available strips)\n";
}

if (defined $options{'local_mode'}) {
	unless (defined $options{'quiet'}) { print STDERR "Operating in local mode\n" }
	open(STDOUT, ">dailystrips-$short_date.html");
	unless (defined $options{'no_index'}) { system("rm -f index.html;ln -s dailystrips-$short_date.html index.html") }

	if (defined $options{'make_archive'}) {
	
		unless (-e "archive.html") { die "Error: archive.html not found" }
		open(ARCHIVE, "<archive.html");
		my @archive = <ARCHIVE>;
		close(ARCHIVE);

		unless (grep(/<a href="dailystrips-$short_date.html">/, @archive)) {
			for (@archive) {
				if ($_ =~ s/(<!--insert below-->)/$1\n<a href="dailystrips-$short_date.html">$long_date<\/a><br>/) {
					open(ARCHIVE, ">archive.html");
					print ARCHIVE @archive;
					close(ARCHIVE);
					last;
				}
			}
		}
	}
}

if (defined $options{'output_file'}) {
	unless (defined $options{'quiet'}) { print STDERR "Writing to file $options{'output_file'}\n" }
	open(STDOUT, ">$options{'output_file'}");
}


# Download image URLs
unless (defined $options{'quiet'}) { print STDERR "Retrieving URLS..." }
for (@get) {
	&get_strip($_);
}
unless (defined $options{'quiet'}) { print STDERR "done\n" }


# Generate HTML page
print <<END_HEADER;
<html>

<head>
	<title>dailystrips for $long_date</title>
</head>

<body bgcolor=\"#ffffff\" text=\"#000000\" link=\"#ff00ff\">

<center>
	<font face=\"helvetica\" size=\"+2\"><b><u>dailystrips for $long_date</u></b></font>
</center>

<table border=\"0\">
END_HEADER

#"#kwrite's syntax higlighting is buggy..

if ((defined $options{'local_mode'}) and (not defined $options{'quiet'})) { print STDERR "Saving strips locally..." }

for (@strips) {
	my ($strip, $homepage, $img_addr) = split(/;/, $_);
	my ($img_line, $local_name, $image);
	
	if ($img_addr =~ "^unavail") {
		$img_line = "[Error - unable to retrieve URL]";
	} else {
		if (defined $options{'local_mode'}) {
			# local mode - download strips
			$img_addr =~ m/(.*)\.(.*)$/o;
			$local_name = "$strip-$short_date.$2";
			
			if (defined $options{'save_existing'} and  -e $local_name) {
					# strip already exists - skip download
					$img_addr = $local_name;
					$img_addr =~ s/ /\%20/go;
					$img_line = "<img src=\"$img_addr\" alt=\"$strip\">";
			} else {
			   # need to download
				$image = &http_get($img_addr);
				if ($image =~ m/^ERROR/o) {
					$img_line = "[Error - unable to download image]";
				} else {
					open(IMAGE, ">$local_name");
					print IMAGE $image;
					close(IMAGE);
			
					$img_addr = $local_name;
					$img_addr =~ s/ /\%20/go;
					$img_line = "<img src=\"$img_addr\" alt=\"$strip\">";
				}
			}
		} else {
			# regular mode - just give addresses to strips on their webserver
			$img_line = "<img src=\"$img_addr\" alt=\"$strip\">";
		}
	}
		
	print <<END_STRIP;
	<tr>
		<td>
			<font face=\"helvetica\" size=\"+1\"><b><a href=\"$homepage\">$strip</a></b></font>
		</td>
	</tr>
	<tr>
		<td>
			$img_line
			<p>&nbsp;</p>
		</td>
	</tr>
END_STRIP
}
#"#kwrite's syntax highlighting is buggy..


if ((defined $options{'local_mode'}) and (not defined $options{'quiet'})) { print STDERR "done\n" }

print <<END_FOOTER;
</table>

<font face=\"helvetica\">Generated by dailystrips $version, by <a href=\"mailto:amedico\@calug.net\">Andrew Medico</a></font>

</body>

</html>
END_FOOTER

#/# kwrite's syntax highlighting is a bit off.. this fixes things

sub http_get {
	my ($url) = @_;
	
	my $request = HTTP::Request->new('GET', $url);
	my $ua = LWP::UserAgent->new;
	$ua->agent("dailystrips $version: " . $ua->agent());
	$ua->env_proxy();

	my $response = $ua->request($request);
	(my $status = $response->status_line()) =~ s/^(\d+)/$1:/;

	if ($response->is_error()) {
		return "ERROR: $status";
	} else {
		return $response ->content;
	}
}

sub get_strip {
	my ($strip) = @_;
	my ($page, $addr);

	# Adam@Home
	if ($strip =~ m/adamathome/io or $strip =~ m/all/io) {
		my $date = strftime("\%Y\/ad\%y%m%d",@localtime);
		push(@strips,"Adam\@Home;http://www.ucomics.com/adamathome/viewad.htm;http://images.ucomics.com/comics/ad/$date.gif")
	}

	# Badtech
	if ($strip =~ m/badtech/io or $strip =~ m/all/io) {
		my $date = strftime("\%Y/%-m/%-d", @localtime);
		$date =~ m/^(\d*?)\//;
		my $year = $1 - 2000;
		$date =~ s/^(\d*?)\//$year\//;

		push(@strips, "Badtech;http://www.badtech.com/;http://www.badtech.com/a/$date.jpg");
	}
	
	# Baldo
	if ($strip =~ m/baldo/io or $strip =~ m/all/io) {
		my $date = strftime("\%Y\/ba\%y%m%d",@localtime);
		push(@strips,"Baldo;http://www.ucomics.com/baldo/viewba.htm;http://images.ucomics.com/comics/ba/$date.gif")
	}

	# Bizarro
	if ($strip =~ m/bizarro/io or $strip =~ m/all/io) {
		my $date = strftime("\%Y\/bz\%y%m%d",@localtime);
		push(@strips,"Bizarro;http://www.ucomics.com/bizarro/viewbz.htm;http://images.ucomics.com/comics/bz/$date.gif")
	}

	# The Big Picture
	if ($strip =~ m/thebigpicture/io or $strip =~ m/all/io) {
		my $date = strftime("\%Y\/bi\%y%m%d",@localtime);
		push(@strips,"The Big Picture;http://www.ucomics.com/thebigpicture/viewbi.htm;http://images.ucomics.com/comics/bi/$date.gif")
	}

	# The Boondocks
	if ($strip =~ m/boondocks/io or $strip =~ m/all/io) {
		my $date = strftime("\%Y\/bo\%y%m%d",@localtime);
		push(@strips,"The Boondocks;http://www.ucomics.com/boondocks/viewbo.htm;http://images.ucomics.com/comics/bo/$date.gif")
	}

	# Calvin and Hobbes - ucomics.com, but isn't updated daily, so fetch current strip from website instead of generating the URL
	if ($strip =~ m/calvinandhobbes/io or $strip =~ m/all/io) {
		$page = &http_get("http://www.ucomics.com/calvinandhobbes/viewch.htm");

		if ($page =~ m/^ERROR/) {
			$addr = "unavail-server";
		} else {
			$page =~ m/<IMG(.*)images.ucomics.com\/comics\/ch\/((\d+)\/ch(\d+)).gif\"(.*)>/i;

			if ($2 eq "") {
				$addr = "unavail-nomatch";
			} else {
				$addr = "http://images.ucomics.com/comics/ch/$2.gif";
			}
		}

		push(@strips, "Calvin and Hobbes;http://www.ucomics.com/calvinandhobbes/viewch.htm;$addr");
	}

	# Cathy
	if ($strip =~ m/cathy/io or $strip =~ m/all/io) {
		my $date = strftime("\%Y\/ca\%y%m%d",@localtime);
		push(@strips,"Cathy;http://www.ucomics.com/cathy/viewca.htm;http://images.ucomics.com/comics/ca/$date.gif")
	}

	# Citizen Dog
	if ($strip =~ m/citizendog/io or $strip =~ m/all/io) {
		my $date = strftime("\%Y\/cd\%y%m%d",@localtime);
		push(@strips,"Citizen Dog;http://www.ucomics.com/citizendog/viewcd.htm;http://images.ucomics.com/comics/cd/$date.gif")
	}

	# Close to Home
	if ($strip =~ m/closetohome/io or $strip =~ m/all/io) {
		my $date = strftime("\%Y\/cl\%y%m%d",@localtime);
		push(@strips,"Close to Home;http://www.ucomics.com/closetohome/viewcl.htm;http://images.ucomics.com/comics/cl/$date.gif")
	}

	# Cornered
	if ($strip =~ m/cornered/io or $strip =~ m/all/io) {
		my $date = strftime("\%Y\/co\%y%m%d",@localtime);
		push(@strips,"Cornered;http://www.ucomics.com/cornered/viewco.htm;http://images.ucomics.com/comics/co/$date.gif")
	}

	# Crabby Road
	if ($strip =~ m/crabbyroad/io or $strip =~ m/all/io) {
		my $date = strftime("\%Y\/cr\%y%m%d",@localtime);
		push(@strips,"Crabby Road;http://www.ucomics.com/crabbyroad/viewcr.htm;http://images.ucomics.com/comics/cr/$date.gif")
	}

	# Crankshaft
	if ($strip =~ m/crankshaft/io or $strip =~ m/all/io) {
		my $date = strftime("\%Y\/cs\%y%m%d",@localtime);
		push(@strips,"Crankshaft;http://www.ucomics.com/crankshaft/viewcs.htm;http://images.ucomics.com/comics/cs/$date.gif")
	}

	# Dilbert
	if ($strip =~ m/dilbert/io or $strip =~ m/all/io) {
		my $date = strftime("\%Y\%m\%d", @localtime);

		$page = &http_get("http://www.dilbert.com/comics/dilbert/archive/dilbert-$date.html");
		
		if ($page =~ m/^ERROR/) {
			$addr = "unavail-server";
		} else {
			$page =~ m/<IMG(.*)SRC="\/comics\/dilbert\/archive\/images\/(.*).gif"/i;

			if ($2 eq "") {
				$addr = "unavail-nomatch";
			} else {
				$addr = "http://www.dilbert.com/comics/dilbert/archive/images/$2.gif";
			}
		}
		
		push(@strips, "Dilbert;http://www.dilbert.com/;$addr");
	}
	
	# Doonesbury
	if ($strip =~ m/doonesbury/io or $strip =~ m/all/io) {
		my $date = strftime("\%Y\/db\%y%m%d",@localtime);
		push(@strips,"Doonesbury;http://www.ucomics.com/doonesbury/viewdb.htm;http://images.ucomics.com/comics/db/$date.gif")
	}

	# Duplex
	if ($strip =~ m/duplex/io or $strip =~ m/all/io) {
		my $date = strftime("\%Y\/dp\%y%m%d",@localtime);
		push(@strips,"Duplex;http://www.ucomics.com/duplex/viewdp.htm;http://images.ucomics.com/comics/dp/$date.gif")
	}

	# The 5th Wave - ucomics.com, but isn't updated daily, so fetch current strip from website instead of generating the URL
	if ($strip =~ m/thefifthwave/io or $strip =~ m/all/io) {
		$page = &http_get("http://www.ucomics.com/thefifthwave/viewfw.htm");
		
		if ($page =~ m/^ERROR/) {
			$addr = "unavail-server";
		} else {
			$page =~ m/<IMG(.*)images.ucomics.com\/comics\/fw\/((\d+)\/fw(\d+)).gif\"(.*)>/i;

			if ($2 eq "") {
				$addr = "unavail-nomatch";
			} else {
				$addr = "http://images.ucomics.com/comics/fw/$2.gif";
			}
		}
		
		push(@strips, "The 5th Wave;http://www.ucomics.com/thefifthwave/viewfw.htm;$addr");
	}

	# FoxTrot
	if ($strip =~ m/foxtrot/io or $strip =~ m/all/io) {
		my $date = strftime("\%Y\/ft\%y%m%d",@localtime);
		push(@strips,"FoxTrot;http://www.ucomics.com/foxtrot/viewft.htm;http://images.ucomics.com/comics/ft/$date.gif")
	}

	# The Fusco Brothers
	if ($strip =~ m/thefuscobrothers/io or $strip =~ m/all/io) {
		my $date = strftime("\%Y\/fu\%y%m%d",@localtime);
		push(@strips,"The Fusco Brothers;http://www.ucomics.com/thefuscobrothers/viewfu.htm;http://images.ucomics.com/comics/fu/$date.gif")
	}

	# Garfield
	if ($strip =~ m/garfield/io or $strip =~ m/all/io) {
		my $date = strftime("\%Y\/ga\%y%m%d",@localtime);
		push(@strips,"Garfield;http://www.ucomics.com/garfield/viewga.htm;http://images.ucomics.com/comics/ga/$date.gif")
	}

	# Heart of the City
	if ($strip =~ m/heartofthecity/io or $strip =~ m/all/io) {
		my $date = strftime("\%Y\/hc\%y%m%d",@localtime);
		push(@strips,"Heart of the City;http://www.ucomics.com/heartofthecity/viewhc.htm;http://images.ucomics.com/comics/hc/$date.gif")
	}

	# In the Bleachers
	if ($strip =~ m/inthebleachers/io or $strip =~ m/all/io) {
		my $date = strftime("\%Y\/bl\%y%m%d",@localtime);
		push(@strips,"In the Bleachers;http://www.ucomics.com/inthebleachers/viewbl.htm;http://images.ucomics.com/comics/bl/$date.gif")
	}

	# James
	if ($strip =~ m/james/io or $strip =~ m/all/io) {
		my $date = strftime("\%Y\/jm\%y%m%d",@localtime);
		push(@strips,"James;http://www.ucomics.com/james/viewjm.htm;http://images.ucomics.com/comics/jm/$date.gif")
	}

	# The Joy of Tech
	if ($strip =~ m/joyoftech/io or $strip =~ m/all/io) {
		$page = &http_get("http://www.joyoftech.com/joyoftech/");

		if ($page =~ m/^ERROR/) {
			$addr = "unavail-server";
		} else {
			$page =~ m/<IMG(.*)src=\"joyimages\/(\d+).gif\"(.*)>/i;

			if ($2 eq "") {
				$addr = "unavail-nomatch";
			} else {
				$addr = "http://www.joyoftech.com/joyoftech/joyimages/$2.gif";
			}
		}

		push(@strips, "The Joy of Tech;http://www.joyoftech.com/joyoftech/;$addr");
	}
	
	# Mister Boffo
	if ($strip =~ m/misterboffo/io or $strip =~ m/all/io) {
		my $date = strftime("\%Y\/mb\%y%m%d",@localtime);
		push(@strips,"Mister Boffo;http://www.ucomics.com/misterboffo/viewmb.htm;http://images.ucomics.com/comics/mb/$date.gif")
	}

	# Non Sequitur
	if ($strip =~ m/nonsequitur/io or $strip =~ m/all/io) {
		my $date = strftime("\%Y\/nq\%y%m%d",@localtime);
		push(@strips,"Non Sequitur;http://www.ucomics.com/nonsequitur/viewnq.htm;http://images.ucomics.com/comics/nq/$date.gif")
	}

	# Oddly Enough
	if ($strip =~ m/oddlyenough/io or $strip =~ m/all/io) {
		my $date = strftime("\%Y\/oe\%y%m%d",@localtime);
		push(@strips,"Oddly Enough;http://www.ucomics.com/oddlyenough/viewoe.htm;http://images.ucomics.com/comics/oe/$date.gif")
	}

	# Overboard
	if ($strip =~ m/overboard/io or $strip =~ m/all/io) {
		my $date = strftime("\%Y\/ob\%y%m%d",@localtime);
		push(@strips,"Overboard;http://www.ucomics.com/overboard/viewob.htm;http://images.ucomics.com/comics/ob/$date.gif")
	}

   	# Penny Arcade
	if ($strip =~ m/pennyarcade/io or $strip =~ m/all/io) {
		$page = &http_get("http://www.penny-arcade.com/view.php3");

		if ($page =~ m/^ERROR/) {
			$addr = "unavail-server";
		} else {
			$page =~ m/<IMG(.*)src=\"images\/(.*).gif\"><BR>/i;

			if ($2 eq "") {
				$addr = "unavail-nomatch";
			} else {
				$addr = "http://www.penny-arcade.com/images/$2.gif";
			}
		}

		push(@strips, "Penny\ Arcade;http://www.penny-arcade.com/;$addr");
	}	
	
	# Real Life Adventures
	if ($strip =~ m/reallifeadventures/io or $strip =~ m/all/io) {
		my $date = strftime("\%Y\/rl\%y%m%d",@localtime);
		push(@strips,"Real Life Adventures;http://www.ucomics.com/reallifeadventures/viewrl.htm;http://images.ucomics.com/comics/rl/$date.gif")
	}

	# Redmeat
	if ($strip =~ m/redmeat/io or $strip =~ m/all/io) {
		push(@strips, "Redmeat;http://www.redmeat.com/redmeat/current/;http://www.redmeat.com/redmeat/current/index-1.gif");
	}
	
	# Reynolds Unwrapped
	if ($strip =~ m/reynoldsunwrapped/io or $strip =~ m/all/io) {
		my $date = strftime("\%Y\/rw\%y%m%d",@localtime);
		push(@strips,"Reynolds Unwrapped;http://www.ucomics.com/reynoldsunwrapped/viewrw.htm;http://images.ucomics.com/comics/rw/$date.gif")
	}

	# Sinfest
	if ($strip =~ m/sinfest/io or $strip =~ m/all/io) {
		my $date = strftime("\%Y\%m\%d", @localtime);
		push(@strips, "Sinfest;http://sinfest.net/;http://sinfest.net/comics/sf$date.gif");
	}
	
	# Stone Soup
	if ($strip =~ m/stonesoup/io or $strip =~ m/all/io) {
		my $date = strftime("\%Y\/ss\%y%m%d",@localtime);
		push(@strips,"Stone Soup;http://www.ucomics.com/stonesoup/viewss.htm;http://images.ucomics.com/comics/ss/$date.gif")
	}

	# Tank McNamara
	if ($strip =~ m/tankmcnamara/io or $strip =~ m/all/io) {
		my $date = strftime("\%Y\/tm\%y%m%d",@localtime);
		push(@strips,"Tank McNamara;http://www.ucomics.com/tankmcnamara/viewtm.htm;http://images.ucomics.com/comics/tm/$date.gif")
	}

	# Tom the Dancing Bug
	if ($strip =~ m/tomthedancingbug/io or $strip =~ m/all/io) {
		my $date = strftime("\%Y\/td\%y%m%d",@localtime);
		push(@strips,"Tom the Dancing Bug;http://www.ucomics.com/tomthedancingbug/viewtd.htm;http://images.ucomics.com/comics/td/$date.gif")
	}

	# UserFriendly
	if ($strip =~ m/userfriendly/io or $strip =~ m/all/io) {
		$page = &http_get("http://www.userfriendly.org/static/");
		
		if ($page =~ m/^ERROR/) {
			$addr = "unavail-server";
		} else {
			$page =~ m/<img(.*?)alt="latest strip"(.*?)src="(.*?)"/i;

			if ($3 eq "") {
				$addr = "unavail-nomatch";
			} else {
				$addr = "$3";
			}
		}
		
		push(@strips, "User\ Friendly;http://www.userfriendly.org/static;$addr");
	}	
	
	# Ziggy
	if ($strip =~ m/ziggy/io or $strip =~ m/all/io) {
		my $date = strftime("\%Y\/zi\%y%m%d",@localtime);
		push(@strips,"Ziggy;http://www.ucomics.com/ziggy/viewzi.htm;http://images.ucomics.com/comics/zi/$date.gif")
	}
}